<?php

/*
 * This file is part of the Tajine software.
 * <https://gitlab.adullact.net/adullact/pki/tajine>
 *
 * Copyright (c) ADULLACT   <https://adullact.org>
 *               Association des Développeurs et Utilisateurs de Logiciels Libres
 *               pour les Administrations et les Collectivités Territoriales
 *
 * Tajine is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this software. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 */

declare(strict_types=1);

namespace App\Tests\Unit\Domain\Certificate\Cfssl;

use PHPUnit\Framework\TestCase;
use App\Domain\Certificate\Cfssl\CfsslInfo;

class CfsslInfoTest extends TestCase
{
    public function testCheckDefaultCfsslProperties(): void
    {
        $cfsslInfo = new CfsslInfo();
        self::AssertSame("http://127.0.0.1:8888", $cfsslInfo->cfsslServer);
        self::AssertSame("info", $cfsslInfo->cfsslApiEndpoint);
        self::AssertSame("api/v1/cfssl", $cfsslInfo->cfsslApiRelativeUrl);
        self::AssertSame("http://127.0.0.1:8888/api/v1/cfssl/info", $cfsslInfo->cfsslApiMethodUrl);
    }

    public function testCheckCustomCfsslProperties(): void
    {
        $cfsslInfo = new CfsslInfo(
            cfsslServer: "http://127.0.0.1:999",
            cfsslApiEndpoint: "not-valid-endpoint",
            cfsslApiRelativeUrl: "api/v4/cfssl",
        );
        self::AssertSame("http://127.0.0.1:999", $cfsslInfo->cfsslServer);
        self::AssertSame("not-valid-endpoint", $cfsslInfo->cfsslApiEndpoint);
        self::AssertSame("api/v4/cfssl", $cfsslInfo->cfsslApiRelativeUrl);
        self::AssertSame("http://127.0.0.1:999/api/v4/cfssl/not-valid-endpoint", $cfsslInfo->cfsslApiMethodUrl);
    }

    public function testTryToRequestBadCfsslServer(): void
    {
        $badCfssServer = "http://127.0.0.1:9999";
        $cfsslInfo = new CfsslInfo(cfsslServer: "$badCfssServer");
        $this->expectExceptionMessage(
            "HTTP request failed - Bad CFSSL URL or CFSSL server is not up [ $badCfssServer/api/v1/cfssl/info ]"
        );
        $cfsslInfo->loadApiRequest();
    }

    public function testTryToRequestBadCfsslEnpoint(): void
    {
        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: "not-valid-endpoint");
        $url = $cfsslInfo->cfsslApiMethodUrl;
        $this->expectExceptionMessage("HTTP 404 Not Found --> Bad CFSSL endpoind [ $url ]");
        $cfsslInfo->loadApiRequest();
    }

    public function testTryToRequestWithNoPaylod(): void
    {
        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: "info");
        $this->expectExceptionMessage("Error HTTP 400 Bad Request --> Json payload is not valid");
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: '');

        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: "newcert");
        $this->expectExceptionMessage("Error HTTP 400 Bad Request --> Json payload is not valid");
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: '');
    }

    public function testTryToRequestWithInvalidJsonPayload(): void
    {
        $invalidJsonPayload = '{"request": { , }';

        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: "info");
        $this->expectExceptionMessage("Error HTTP 400 Bad Request --> Json payload is not valid");
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: "$invalidJsonPayload");

        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: "newcert");
        $this->expectExceptionMessage("Error HTTP 400 Bad Request --> Json payload is not valid");
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: "$invalidJsonPayload");
    }


    public function testValidCfsslRequestToInfoEndpoint(): void
    {
        $endpoint = 'info';
        $cfsslInfo = new CfsslInfo(cfsslApiEndpoint: $endpoint);
//      $cfsslApiJsonPayload = '';                  // invalid payload
//      $cfsslApiJsonPayload = '{"request": { , }'; // invalid json payload
        $cfsslApiJsonPayload = '{}';
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: $cfsslApiJsonPayload);

//      dump($cfsslInfo->cfsslApiJsonResponse);
//      dump($cfsslInfo->cfsslApiJsonPayload);
//      dump($cfsslInfo->curlCode);
//      $fileBase = "CFSSL_API_$endpoint.error-HTTP-400.error-CFSSL-2400-key-size-to-large";
//      $fileBase = "CFSSL_API_$endpoint.bad-http-method";
//      $fileBase = "CFSSL_API_$endpoint.no-payload";
//      $fileBase = "CFSSL_API_$endpoint.empty-payload";
//      $fileBase = "CFSSL_API_$endpoint";
//      $fileBase = "CFSSL_API_$endpoint.invalid-json-payload";
//      file_put_contents(filename: "$fileBase.response.json" , data: $cfsslInfo->cfsslApiJsonResponse);
//      file_put_contents(filename: "$fileBase.payload.json" , data: $cfsslInfo->cfsslApiJsonPayload);
    }


    public function testValidCfsslRequestToNewcertEndpoint(): void
    {
//      $endpoint = 'info';
        $endpoint = 'newcert';
        $cfsslInfo = new CfsslInfo(
            cfsslApiEndpoint: $endpoint,
        );
        $userMail = "user@example.org";
        $orgName = "Mairie de Montpellier";
        $cfsslApiJsonPayload = '
        {  "request": {
            "hosts": [""],
            "key": { "algo":  "rsa", "size": 2048 },
            "names": [{  "C": "FR",
                        "ST": "Herault",
                        "L": "Montpellier",
                        "O": "' . $orgName . '",
                        "OU": "465146154365143651"    }],
            "CN": "' . $userMail . '"
          }
        }';
//      $cfsslApiJsonPayload = '';
//      $cfsslApiJsonPayload = '{}';
//      $cfsslApiJsonPayload = '{"request": { , }'; // invalid json payload
        $cfsslInfo->loadApiRequest(cfsslApiJsonPayload: $cfsslApiJsonPayload);

      dump($cfsslInfo->cfsslApiJsonResponse);
      dump($cfsslInfo->cfsslApiJsonPayload);
//      dump($cfsslInfo->curlCode);
//      $fileBase = "CFSSL_API_$endpoint.error-HTTP-400.error-CFSSL-2400-key-size-to-large";
//      $fileBase = "CFSSL_API_$endpoint.bad-http-method";
//      $fileBase = "CFSSL_API_$endpoint.no-payload";
//      $fileBase = "CFSSL_API_$endpoint.empty-payload";
//      $fileBase = "CFSSL_API_$endpoint";
//      $fileBase = "CFSSL_API_$endpoint.invalid-json-payload";
//      file_put_contents(filename: "$fileBase.response.json" , data: $cfsslInfo->cfsslApiJsonResponse);
//      file_put_contents(filename: "$fileBase.payload.json" , data: $cfsslInfo->cfsslApiJsonPayload);
    }


}
